﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using Gurock.SmartInspect;

namespace DojoNorthSoftware.WindowsServerSolutions.HomeServerSMART2013.Components
{
    public sealed class IdentifySsdManufacturer
    {
        public static string SOLID_STATE_DISK_GLOBAL_ID = "Sl0bberhe@d";

        public static SsdManufacturer GetSsdManufacturerFromSmartAttributes(byte[] smartData)
        {
            SiAuto.Main.EnterMethod("HomeServerSMART2013.Components.IdentifySsdManufacturer.GetSsdManufacturerFromSmartAttributes");
            int[] attributeList = new int[30];

            for (int i = 0; i < 30; i++)
            {
                int offset = 12 * i;

                // Attribute ID: 2
                attributeList[i] = smartData[offset + 2];
                if (attributeList[i] == 0)
                {
                    // Done!
                    break;
                }
            }

            // Must do Micron before SandForce because SandForce and Micron all start 1, 5, 9, 12, 170; this would cause
            // a Micron controller to be registered as a SandForce. Micron has 171, 172, 173 and more so get it first.
            // No overlap on other controllers so we can process them in any order. They're listed below in the order of
            // likelihood for best performance, at least SandForce, Intel and Samsung.
            if (IsSsdMicron(attributeList))
            {
                SiAuto.Main.LogColored(System.Drawing.Color.LightBlue, "[SSD Identification] Controller identified as Micron.");
                SiAuto.Main.LeaveMethod("HomeServerSMART2013.Components.IdentifySsdManufacturer.GetSsdManufacturerFromSmartAttributes");
                return SsdManufacturer.SSD_MANUFACTURER_MICRON;
            }
            if (IsSsdLamd(attributeList))
            {
                SiAuto.Main.LogColored(System.Drawing.Color.LightBlue, "[SSD Identification] Controller identified as LAMD.");
                SiAuto.Main.LeaveMethod("HomeServerSMART2013.Components.IdentifySsdManufacturer.GetSsdManufacturerFromSmartAttributes");
                return SsdManufacturer.SSD_MANUFACTURER_LAMD;
            }
            if (IsSsdSandForce(attributeList))
            {
                SiAuto.Main.LogColored(System.Drawing.Color.LightBlue, "[SSD Identification] Controller identified as SandForce.");
                SiAuto.Main.LeaveMethod("HomeServerSMART2013.Components.IdentifySsdManufacturer.GetSsdManufacturerFromSmartAttributes");
                return SsdManufacturer.SSD_MANUFACTURER_SANDFORCE;
            }
            else if (IsSsdIntel(attributeList))
            {
                SiAuto.Main.LogColored(System.Drawing.Color.LightBlue, "[SSD Identification] Controller identified as Intel.");
                SiAuto.Main.LeaveMethod("HomeServerSMART2013.Components.IdentifySsdManufacturer.GetSsdManufacturerFromSmartAttributes");
                return SsdManufacturer.SSD_MANUFACTURER_INTEL;
            }
            else if (IsSsdSamsung(attributeList))
            {
                SiAuto.Main.LogColored(System.Drawing.Color.LightBlue, "[SSD Identification] Controller identified as Samsung.");
                SiAuto.Main.LeaveMethod("HomeServerSMART2013.Components.IdentifySsdManufacturer.GetSsdManufacturerFromSmartAttributes");
                return SsdManufacturer.SSD_MANUFACTURER_SAMSUNG;
            }
            else if (IsSsdIndilinx(attributeList))
            {
                SiAuto.Main.LogColored(System.Drawing.Color.LightBlue, "[SSD Identification] Controller identified as Indilinx.");
                SiAuto.Main.LeaveMethod("HomeServerSMART2013.Components.IdentifySsdManufacturer.GetSsdManufacturerFromSmartAttributes");
                return SsdManufacturer.SSD_MANUFACTURER_INDILINX;
            }
            else if (IsSsdEverest(attributeList))
            {
                SiAuto.Main.LogColored(System.Drawing.Color.LightBlue, "[SSD Identification] Controller identified as Indilinx Everest.");
                SiAuto.Main.LeaveMethod("HomeServerSMART2013.Components.IdentifySsdManufacturer.GetSsdManufacturerFromSmartAttributes");
                return SsdManufacturer.SSD_MANUFACTURER_EVEREST;
            }
            else if (IsSsdBarefoot3(attributeList))
            {
                SiAuto.Main.LogColored(System.Drawing.Color.LightBlue, "[SSD Identification] Controller identified as Indilinx Barefoot 3.");
                SiAuto.Main.LeaveMethod("HomeServerSMART2013.Components.IdentifySsdManufacturer.GetSsdManufacturerFromSmartAttributes");
                return SsdManufacturer.SSD_MANUFACTURER_BAREFOOT3;
            }
            else if (IsSsdJMicron(attributeList))
            {
                SiAuto.Main.LogColored(System.Drawing.Color.LightBlue, "[SSD Identification] Controller identified as JMicron.");
                SiAuto.Main.LeaveMethod("HomeServerSMART2013.Components.IdentifySsdManufacturer.GetSsdManufacturerFromSmartAttributes");
                return SsdManufacturer.SSD_MANUFACTURER_JMICRON;
            }
            else if (IsSsdMarvell(attributeList))
            {
                SiAuto.Main.LogColored(System.Drawing.Color.LightBlue, "[SSD Identification] Controller identified as Marvell.");
                SiAuto.Main.LeaveMethod("HomeServerSMART2013.Components.IdentifySsdManufacturer.GetSsdManufacturerFromSmartAttributes");
                return SsdManufacturer.SSD_MANUFACTURER_MARVELL;
            }
            else if (IsSsdSmartModular(attributeList))
            {
                SiAuto.Main.LogColored(System.Drawing.Color.LightBlue, "[SSD Identification] Controller identified as SMART Modular.");
                SiAuto.Main.LeaveMethod("HomeServerSMART2013.Components.IdentifySsdManufacturer.GetSsdManufacturerFromSmartAttributes");
                return SsdManufacturer.SSD_MANUFACTURER_SMART_MOD;
            }
            else if (IsSsdStec(attributeList))
            {
                SiAuto.Main.LogColored(System.Drawing.Color.LightBlue, "[SSD Identification] Controller identified as Stec.");
                SiAuto.Main.LeaveMethod("HomeServerSMART2013.Components.IdentifySsdManufacturer.GetSsdManufacturerFromSmartAttributes");
                return SsdManufacturer.SSD_MANUFACTURER_STEC;
            }
            else if (IsSsdLamd(attributeList))
            {
                SiAuto.Main.LogColored(System.Drawing.Color.LightBlue, "[SSD Identification] Controller identified as LAMD.");
                SiAuto.Main.LeaveMethod("HomeServerSMART2013.Components.IdentifySsdManufacturer.GetSsdManufacturerFromSmartAttributes");
                return SsdManufacturer.SSD_MANUFACTURER_LAMD;
            }

            // Didn't match to anything, so handle as a generic.
            SiAuto.Main.LogColored(System.Drawing.Color.LightBlue, "[SSD Identification] Controller identified as Unknown/Generic.");
            SiAuto.Main.LeaveMethod("HomeServerSMART2013.Components.IdentifySsdManufacturer.GetSsdManufacturerFromSmartAttributes");
            return SsdManufacturer.SSD_MANUFACTURER_OTHER;
        }

        /// <summary>
        /// Returns whether or not an SSD uses a SandForce controller based on its SMART data.
        /// </summary>
        /// <param name="attributeList">List of attribute IDs.</param>
        /// <returns>true if SandForce; false otherwise.</returns>
        private static bool IsSsdSandForce(int[] attributeList)
        {
            SiAuto.Main.EnterMethod("HomeServerSMART2013.Components.IdentifySsdManufacturer.IsSsdSandForce");
            if (attributeList[0] == 1 && attributeList[1] == 5 && attributeList[2] == 9 &&
                attributeList[3] == 12 && attributeList[4] == 170)
            {
                SiAuto.Main.LogMessage("[SSD Identification] Pattern match on attributes 1, 5, 9, 12, 170 - SandForce");
                SiAuto.Main.LeaveMethod("HomeServerSMART2013.Components.IdentifySsdManufacturer.IsSsdSandForce");
                return true;
            }
            if (attributeList[0] == 1 && attributeList[1] == 5 && attributeList[2] == 9 &&
                attributeList[3] == 12 && attributeList[4] == 171 && attributeList[5] == 172)
            {
                SiAuto.Main.LogMessage("[SSD Identification] Pattern match on attributes 1, 5, 9, 12, 171, 172 - SandForce");
                SiAuto.Main.LeaveMethod("HomeServerSMART2013.Components.IdentifySsdManufacturer.IsSsdSandForce");
                return true;
            }
            SiAuto.Main.LeaveMethod("HomeServerSMART2013.Components.IdentifySsdManufacturer.IsSsdSandForce");
            return false;
        }

        /// <summary>
        /// Returns whether or not an SSD uses a LAMD controller based on its SMART data.
        /// </summary>
        /// <param name="attributeList">List of attribute IDs.</param>
        /// <returns>true if LAMD; false otherwise.</returns>
        private static bool IsSsdLamd(int[] attributeList)
        {
            SiAuto.Main.EnterMethod("HomeServerSMART2013.Components.IdentifySsdManufacturer.IsSsdLamd");
            if (attributeList[0] == 1 && attributeList[1] == 5 && attributeList[2] == 9 &&
                attributeList[3] == 12 && attributeList[4] == 171 && attributeList[5] == 172 &&
                attributeList[6] == 181)
            {
                SiAuto.Main.LogMessage("[SSD Identification] Pattern match on attributes 1, 5, 9, 12, 171, 172, 181 - LAMD");
                SiAuto.Main.LeaveMethod("HomeServerSMART2013.Components.IdentifySsdManufacturer.IsSsdLamd");
                return true;
            }
            if (attributeList[0] == 1 && attributeList[1] == 5 && attributeList[2] == 9 &&
                attributeList[3] == 11 && attributeList[4] == 12 && attributeList[5] == 174)
            {
                SiAuto.Main.LogMessage("[SSD Identification] Pattern match on attributes 1, 5, 9, 11, 12, 174 - LAMD");
                SiAuto.Main.LeaveMethod("HomeServerSMART2013.Components.IdentifySsdManufacturer.IsSsdLamd");
                return true;
            }
            SiAuto.Main.LeaveMethod("HomeServerSMART2013.Components.IdentifySsdManufacturer.IsSsdLamd");
            return false;
        }

        /// <summary>
        /// Returns whether or not an SSD uses an Intel controller based on its SMART data.
        /// </summary>
        /// <param name="attributeList">List of attribute IDs.</param>
        /// <returns>true if Intel; false otherwise.</returns>
        private static bool IsSsdIntel(int[] attributeList)
        {
            SiAuto.Main.EnterMethod("HomeServerSMART2013.Components.IdentifySsdManufacturer.IsSsdIntel");
            if (attributeList[0] == 3 && attributeList[1] == 4 && attributeList[2] == 5 &&
                attributeList[3] == 9 && attributeList[4] == 12 && attributeList[5] == 192 &&
                ((attributeList[6] == 232 && attributeList[7] == 233) || attributeList[6] == 225))
            {
                SiAuto.Main.LogMessage("[SSD Identification] Pattern match on attributes 3, 4, 5, 9, 12, 192, ((232, 233) | 225) - Intel");
                SiAuto.Main.LeaveMethod("HomeServerSMART2013.Components.IdentifySsdManufacturer.IsSsdIntel");
                return true;
            }
            if (attributeList[0] == 3 && attributeList[1] == 4 && attributeList[2] == 5 &&
                attributeList[3] == 9 && attributeList[4] == 12 && attributeList[5] == 170)
            {
                SiAuto.Main.LogMessage("[SSD Identification] Pattern match on attributes 3, 4, 5, 9, 12, 170 - Intel");
                SiAuto.Main.LeaveMethod("HomeServerSMART2013.Components.IdentifySsdManufacturer.IsSsdIntel");
                return true;
            }
            if (attributeList[0] == 5 && attributeList[1] == 9 && attributeList[2] == 12 &&
                attributeList[3] == 170)
            {
                SiAuto.Main.LogMessage("[SSD Identification] Pattern match on attributes 5, 9, 12, 170 - Intel");
                SiAuto.Main.LeaveMethod("HomeServerSMART2013.Components.IdentifySsdManufacturer.IsSsdIntel");
                return true;
            }
            SiAuto.Main.LeaveMethod("HomeServerSMART2013.Components.IdentifySsdManufacturer.IsSsdIntel");
            return false;
        }

        /// <summary>
        /// Returns whether or not an SSD uses a Samsung controller based on its SMART data.
        /// </summary>
        /// <param name="attributeList">List of attribute IDs.</param>
        /// <returns>true if Samsung; false otherwise.</returns>
        private static bool IsSsdSamsung(int[] attributeList)
        {
            SiAuto.Main.EnterMethod("HomeServerSMART2013.Components.IdentifySsdManufacturer.IsSsdSamsung");
            if (attributeList[0] == 5 && attributeList[1] == 9 && attributeList[2] == 12 &&
                attributeList[3] == 177)
            {
                SiAuto.Main.LogMessage("[SSD Identification] Pattern match on attributes 5, 9, 12, 177 - Samsung");
                SiAuto.Main.LeaveMethod("HomeServerSMART2013.Components.IdentifySsdManufacturer.IsSsdSamsung");
                return true;
            }
            if (attributeList[0] == 9 && attributeList[1] == 12 && attributeList[2] == 178 &&
                attributeList[3] == 179 && attributeList[4] == 180)
            {
                SiAuto.Main.LogMessage("[SSD Identification] Pattern match on attributes 9, 12, 178, 179, 180 - Samsung");
                SiAuto.Main.LeaveMethod("HomeServerSMART2013.Components.IdentifySsdManufacturer.IsSsdSamsung");
                return true;
            }
            if (attributeList[0] == 9 && attributeList[1] == 12 && attributeList[2] == 175 &&
                attributeList[3] == 176 && attributeList[4] == 177 && attributeList[5] == 178 &&
                attributeList[6] == 179 && attributeList[7] == 180)
            {
                SiAuto.Main.LogMessage("[SSD Identification] Pattern match on attributes 9, 12, 175, 176, 177, 178, 179, 180 - Samsung");
                SiAuto.Main.LeaveMethod("HomeServerSMART2013.Components.IdentifySsdManufacturer.IsSsdSamsung");
                return true;
            }
            SiAuto.Main.LeaveMethod("HomeServerSMART2013.Components.IdentifySsdManufacturer.IsSsdSamsung");
            return false;
        }

        /// <summary>
        /// Returns whether or not an SSD uses an Indilinx controller based on its SMART data.
        /// </summary>
        /// <param name="attributeList">List of attribute IDs.</param>
        /// <returns>true if Indilinx; false otherwise.</returns>
        private static bool IsSsdIndilinx(int[] attributeList)
        {
            SiAuto.Main.EnterMethod("HomeServerSMART2013.Components.IdentifySsdManufacturer.IsSsdIndilinx");
            if (attributeList[0] == 1 && attributeList[1] == 9 && attributeList[2] == 12 &&
                attributeList[3] == 184)
            {
                SiAuto.Main.LogMessage("[SSD Identification] Pattern match on attributes 1, 9, 12, 184 - Indilinx");
                SiAuto.Main.LeaveMethod("HomeServerSMART2013.Components.IdentifySsdManufacturer.IsSsdIndilinx");
                return true;
            }
            SiAuto.Main.LeaveMethod("HomeServerSMART2013.Components.IdentifySsdManufacturer.IsSsdIndilinx");
            return false;
        }

        /// <summary>
        /// Returns whether or not an SSD uses an Everest (officially Indilinx Everest) controller based on its SMART data.
        /// </summary>
        /// <param name="attributeList">List of attribute IDs.</param>
        /// <returns>true if Indilinx; false otherwise.</returns>
        private static bool IsSsdEverest(int[] attributeList)
        {
            SiAuto.Main.EnterMethod("HomeServerSMART2013.Components.IdentifySsdManufacturer.IsSsdEverest");
            if (attributeList[0] == 1 && attributeList[1] == 3 && attributeList[2] == 4 &&
                attributeList[3] == 5)
            {
                SiAuto.Main.LogMessage("[SSD Identification] Pattern match on attributes 1, 3, 4, 5 - Indilinx Everest");
                SiAuto.Main.LeaveMethod("HomeServerSMART2013.Components.IdentifySsdManufacturer.IsSsdEverest");
                return true;
            }
            SiAuto.Main.LeaveMethod("HomeServerSMART2013.Components.IdentifySsdManufacturer.IsSsdEverest");
            return false;
        }

        /// <summary>
        /// Returns whether or not an SSD uses an Indilinx Barefoot 3 controller based on its SMART data.
        /// </summary>
        /// <param name="attributeList">List of attribute IDs.</param>
        /// <returns>true if Indilinx BF3; false otherwise.</returns>
        private static bool IsSsdBarefoot3(int[] attributeList)
        {
            SiAuto.Main.EnterMethod("HomeServerSMART2013.Components.IdentifySsdManufacturer.IsSsdBarefoot3");
            if (attributeList[0] == 5 && attributeList[1] == 9 && attributeList[2] == 12 &&
                attributeList[3] == 171)
            {
                SiAuto.Main.LogMessage("[SSD Identification] Pattern match on attributes 1, 3, 4, 5 - Indilinx Barefoot 3");
                SiAuto.Main.LeaveMethod("HomeServerSMART2013.Components.IdentifySsdManufacturer.IsSsdBarefoot3");
                return true;
            }
            SiAuto.Main.LeaveMethod("HomeServerSMART2013.Components.IdentifySsdManufacturer.IsSsdBarefoot3");
            return false;
        }

        /// <summary>
        /// Returns whether or not an SSD uses a JMicron controller based on its SMART data.
        /// </summary>
        /// <param name="attributeList">List of attribute IDs.</param>
        /// <returns>true if JMicron; false otherwise.</returns>
        private static bool IsSsdJMicron(int[] attributeList)
        {
            SiAuto.Main.EnterMethod("HomeServerSMART2013.Components.IdentifySsdManufacturer.IsSsdJMicron");
            if (attributeList[0] == 9 && attributeList[1] == 12 && attributeList[2] == 194 &&
                attributeList[3] == 229 && attributeList[4] == 232 && attributeList[5] == 233)
            {
                SiAuto.Main.LogMessage("[SSD Identification] Pattern match on attributes 9, 12, 194, 229, 232, 233 - JMicron");
                SiAuto.Main.LeaveMethod("HomeServerSMART2013.Components.IdentifySsdManufacturer.IsSsdJMicron");
                return true;
            }
            if (attributeList[0] == 1 && attributeList[1] == 2 && attributeList[2] == 3 &&
                attributeList[3] == 5 && attributeList[4] == 7 && attributeList[5] == 8 &&
                attributeList[6] == 9 && attributeList[7] == 10 && attributeList[8] == 12)
            {
                SiAuto.Main.LogMessage("[SSD Identification] Pattern match on attributes 1, 2, 3, 5, 7, 8, 9, 10, 12 - JMicron");
                SiAuto.Main.LeaveMethod("HomeServerSMART2013.Components.IdentifySsdManufacturer.IsSsdJMicron");
                return true;
            }
            SiAuto.Main.LeaveMethod("HomeServerSMART2013.Components.IdentifySsdManufacturer.IsSsdJMicron");
            return false;
        }

        /// <summary>
        /// Returns whether or not an SSD uses a Micron controller based on its SMART data.
        /// </summary>
        /// <param name="attributeList">List of attribute IDs.</param>
        /// <returns>true if Micron; false otherwise.</returns>
        private static bool IsSsdMicron(int[] attributeList)
        {
            SiAuto.Main.EnterMethod("HomeServerSMART2013.Components.IdentifySsdManufacturer.IsSsdMicron");
            if (attributeList[0] == 1 && attributeList[1] == 5 && attributeList[2] == 9 &&
                attributeList[3] == 12 && attributeList[4] == 170 && attributeList[5] == 171 &&
                attributeList[6] == 172 && attributeList[7] == 173 && attributeList[8] == 174 &&
                attributeList[9] == 181 && attributeList[10] == 183)
            {
                SiAuto.Main.LogMessage("[SSD Identification] Pattern match on attributes 1, 5, 9, 12, 170, 171, 172, 173, 174, 181, 183 - Micron");
                SiAuto.Main.LeaveMethod("HomeServerSMART2013.Components.IdentifySsdManufacturer.IsSsdMicron");
                return true;
            }
            SiAuto.Main.LeaveMethod("HomeServerSMART2013.Components.IdentifySsdManufacturer.IsSsdMicron");
            return false;
        }

        /// <summary>
        /// Returns whether or not an SSD uses a SMART Modular controller based on its SMART data.
        /// </summary>
        /// <param name="attributeList">List of attribute IDs.</param>
        /// <returns>true if SMART Modular; false otherwise.</returns>
        private static bool IsSsdSmartModular(int[] attributeList)
        {
            SiAuto.Main.EnterMethod("HomeServerSMART2013.Components.IdentifySsdManufacturer.IsSsdSmartModular");
            if (attributeList[0] == 9 && attributeList[1] == 232 && attributeList[2] == 233 &&
                attributeList[3] == 234 && attributeList[4] == 235)
            {
                SiAuto.Main.LogMessage("[SSD Identification] Pattern match on attributes 9, 232, 233, 234, 235 - SMART Modular");
                SiAuto.Main.LeaveMethod("HomeServerSMART2013.Components.IdentifySsdManufacturer.IsSsdSmartModular");
                return true;
            }
            if (attributeList[0] == 9 && attributeList[1] == 12 && attributeList[2] == 192 &&
                attributeList[3] == 197 && attributeList[4] == 198 && attributeList[5] == 199 &&
                attributeList[6] == 248 && attributeList[7] == 252 && attributeList[8] == 254)
            {
                SiAuto.Main.LogMessage("[SSD Identification] Pattern match on attributes 9, 12, 191, 197, 198, 199, 248, 252, 254 - SMART Modular");
                SiAuto.Main.LeaveMethod("HomeServerSMART2013.Components.IdentifySsdManufacturer.IsSsdSmartModular");
                return true;
            }
            SiAuto.Main.LeaveMethod("HomeServerSMART2013.Components.IdentifySsdManufacturer.IsSsdSmartModular");
            return false;
        }

        /// <summary>
        /// Returns whether or not an SSD uses a STEC controller based on its SMART data.
        /// </summary>
        /// <param name="attributeList">List of attribute IDs.</param>
        /// <returns>true if STEC; false otherwise.</returns>
        private static bool IsSsdStec(int[] attributeList)
        {
            SiAuto.Main.EnterMethod("HomeServerSMART2013.Components.IdentifySsdManufacturer.IsSsdSmartStec");
            if (attributeList[0] == 1 && attributeList[1] == 2 && attributeList[2] == 5 &&
                attributeList[3] == 9 && attributeList[4] == 12 && attributeList[5] == 13 &&
                attributeList[6] == 100)
            {
                SiAuto.Main.LogMessage("[SSD Identification] Pattern match on attributes 1, 2, 5, 9, 12, 13, 100 - STEC");
                SiAuto.Main.LeaveMethod("HomeServerSMART2013.Components.IdentifySsdManufacturer.IsSsdSmartStec");
                return true;
            }
            SiAuto.Main.LeaveMethod("HomeServerSMART2013.Components.IdentifySsdManufacturer.IsSsdSmartStec");
            return false;
        }

        /// <summary>
        /// Returns whether or not an SSD uses a Marvell controller based on its SMART data. (Common in Plextor SSDs)
        /// </summary>
        /// <param name="attributeList">List of attribute IDs.</param>
        /// <returns>true if Marvell; false otherwise.</returns>
        private static bool IsSsdMarvell(int[] attributeList)
        {
            SiAuto.Main.EnterMethod("HomeServerSMART2013.Components.IdentifySsdManufacturer.IsSsdMarvell");
            if (attributeList[0] == 1 && attributeList[1] == 5 && attributeList[2] == 9 &&
                attributeList[3] == 12 && attributeList[4] == 177)
            {
                SiAuto.Main.LogMessage("[SSD Identification] Pattern match on attributes 1, 5, 9, 12, 177 - Marvell");
                SiAuto.Main.LeaveMethod("HomeServerSMART2013.Components.IdentifySsdManufacturer.IsSsdMarvell");
                return true;
            }
            SiAuto.Main.LeaveMethod("HomeServerSMART2013.Components.IdentifySsdManufacturer.IsSsdMarvell");
            return false;
        }
    }

    public enum SsdManufacturer : int
    {
        SSD_MANUFACTURER_SANDFORCE = 0,
        SSD_MANUFACTURER_INTEL = 1,
        SSD_MANUFACTURER_SAMSUNG = 2,
        SSD_MANUFACTURER_MICRON = 3,
        SSD_MANUFACTURER_JMICRON = 4,
        SSD_MANUFACTURER_STEC = 5,
        SSD_MANUFACTURER_SMART_MOD = 6,
        SSD_MANUFACTURER_INDILINX = 7,
        SSD_MANUFACTURER_EVEREST = 8,
        SSD_MANUFACTURER_OTHER = 9,
        SSD_MANUFACTURER_MARVELL = 10,
        SSD_MANUFACTURER_BAREFOOT3 = 11,
        SSD_MANUFACTURER_LAMD = 12,
        SSD_MANUFACTURER_HARD_DISK
    }
}
